#!/bin/csh
#
# precom.s [-d <DIRECTORY> ] [-m <MAKEFILE>] [-e <EXECUTABLE>] \
#          [-l <LIBRARIES>]  <SOURCE_CODE>
#
# Uses the EDITOR executable xedit22 to precompile the source code 
# <SOURCE_CODE>.
#
#============== SCRIPT FILE TO PRECOMPILE A SOURCE CODE ===============#
#
# Set defaults
#
set SOURCE_CODE  = "$argv[$#argv]"
set DIRECTORY    = "${SOURCE_CODE}.dir"
set MAKEFILE     = "${SOURCE_CODE}.make"
set EXECUTABLE   = "${SOURCE_CODE}.x"
set LIBRARIES    = ""
#
# Check for options on the command line.
#
while ($#argv > 1)
  switch ($1)
  case -d:
    set DIRECTORY = "$2"
    breaksw
  case -m:
    set MAKEFILE = "$2"
    breaksw
  case -l:
    set LIBRARIES = "$2"
    breaksw
  case -e:
    set EXECUTABLE = "$2"
    breaksw
  default:
    breaksw
  endsw
  shift
end
#
#======================> If necessary, create the directory "DIRECTORY".
if (! -e $DIRECTORY) mkdir $DIRECTORY
#
#-----------------------> Create the input deck for EDITOR, and execute.
\rm -f inedit
cat << EOF > inedit
 \$editpar   inname='$SOURCE_CODE'
          , ibanner=1, job=4, idump=1, inmlst=1, iutask=0
          , iupdate=1, ext='.f', branch='$DIRECTORY'
          , makename='$MAKEFILE', xeq='$EXECUTABLE'
          , libs='$LIBRARIES' \$
EOF
chmod 700 xedit22
xedit22
