c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 1             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac01 ( namelist, nmax )
c
c    dac:namelist.nlsdac01 <----------------------- reset namelist array
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine initialises all the elements of the array
c  "namelist" to 32 character blanks words.
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*32  blank32
       integer       nmax    , i
c
       character*32  namelist(   2,   *)
c
c      Data statements
c
       data          blank32 /'                                '/
c
c-----------------------------------------------------------------------
c
       do 10 i=1,nmax
         namelist(1,i) = blank32
         namelist(2,i) = blank32
10     continue
c
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 1             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 2             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac02 ( nlunit, cname, namelist, nmax, ival )
c
c    dac:namelist.nlsdac02 <----------------- reads input character data
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1: February, 1991 by David Clarke; Fixed various bugs
c                  discovered in the previous 20 months of use.
c                  Character strings of indefinite length can now be
c                  read.
c
c  PURPOSE:  This subroutine reads the input decks originally designed
c  for CFT "namelist" (under CTSS), and stores the character
c  representations of the input data in "namelist(n,2)".  Meanwhile,
c  "namelist(n,1)" contains a list of the corresponding variable names,
c  which were set by the calling program.  Other subroutines (called by
c  the calling program) convert the character strings to the correct
c  attribute.
c
c  EXTERNALS:  NLSDAC15 - 23
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*1   char1   , char2   , char3
       character*8   cname   , word8
       character*32  blank32 , word32  , temp32
       character*72  line    , oldline
       integer       nlunit  , nmax    , ival    , nlist   , jbeg
     1             , jend    , jran    , jbegold , jendold , i
     2             , i1      , i2      , ilist   , ilisto  , match
     3             , iqte
c
       character*32  namelist(   2,   *)
c
c      Data statements
c
       data blank32 /'                                '/
c
c      External statements
c
       external      nlsdac15, nlsdac16, nlsdac17, nlsdac18, nlsdac19
     1             , nlsdac20, nlsdac21, nlsdac22, nlsdac23
c
c-----------------------------------------------------------------------
c
c      Initialise parameters.  Note that all the second elements of the
c  array "namelist" (namelist(2,*)) and all but the first "ival" of the
c  first elements (namelist(1,*)) are reset to 32 character blanks
c  words.  This is to allow for the possibility that the READ statement
c  appears in a loop, in which case, most of "namelist" must be reset.
c
       do 10 i=1,nmax
         namelist(2,i) = blank32
10     continue
       if (ival .ge. nmax) return
       do 20 i=ival+1,nmax
         namelist(1,i) = blank32
20     continue
       nlist = ival
       iqte  = 0
30     continue
c
c      Read line from input deck.
c
       read (nlunit, 1010, end=150, err=160) line
c
c      Error message if column 1 contains anything but a 'c', 'C', or a
c  blank.
c
       char2 = line(1:1)
       if ( (char2 .ne. 'c') .and. (char2 .ne. 'C') .and.
     1      (char2 .ne. ' ') ) call nlsdac15 ( line, 1, 1 )
c
c      Error message if column 2 contains anything but a '$' or a blank
c  (unless column 1 contains a 'c' or a 'C').
c
       char3 = line(2:2)
       if ( (char2 .ne. 'c') .and. (char2 .ne. 'C') .and.
     1      (char3 .ne. '$') .and. (char3 .ne. ' ') )
     2   call nlsdac15 ( line, 2, 2 )
c
c      If column 1 contains a 'c' or a 'C', the line is "commented out",
c  and is echoed on the terminal.
c
       if ( (char2 .eq. 'c') .or. (char2 .eq. 'C') ) then
         write (6, 2010) line
         go to 30
       endif
c
c      If column 2 contains a '$' sentinal, then a new namelist begins.
c  Determine if name of namelist matches "cname", and if it does, read
c  the contents of the namelist.  The next '$' sentinal will close the
c  namelist.
c
       if (char3 .eq. '$') then
         jbeg = 3
         call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
         if (char1 .eq. '$') go to 30
         call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte, word32 )
         word8 = word32(1:8)
         if (word8 .eq. cname) then
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           if (char1 .eq. '$') return
40         continue
           i1 = 0
           i2 = 0
c
c      Read the next specified variable.
c
           call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                   , word32 )
           do 50 i=1,nmax
             ilist  = i
             temp32 = blank32
             temp32(1:8) = namelist(1,ilist)(1:8)
             if (temp32 .eq. blank32) go to 60
             if (temp32 .eq.  word32) then
               if (namelist(2,ilist) .ne. blank32) then
                 temp32 = namelist(1,ilist)
                 nlist  = nlist + 1
                 ilist  = nlist
                 namelist(1,ilist)( 1: 8) = temp32( 1: 8)
                 namelist(1,ilist)(25:32) = temp32(25:32)
               endif
               go to 70
             endif
50         continue
c
c      Error:  variable specified in input deck was not included in
c  "namelist".
c
60         continue
           call nlsdac15 ( line, jbeg, 3 )
c
c      Specified variable found in "namelist".  Read the next character.
c  If "char1" .ne. '(' or '=', issue error message.  If "char1" = '(', a
c  portion of an array will be set.  If "char1" = '=', the variable valu
c  will be read next.
c
70         continue
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           match = 1
           if (char1 .eq. '(') match = 2
           if (char1 .eq. '=') match = 3
           go to (80, 90, 130) match
c
c      Error: unexpected character.
c
80         continue
           call nlsdac15 ( line, jbeg, 4 )
c
c      Specify portion of vector or array to be set.  The limits are
c  stored in columns 10 through 28  of "namelist(1,ilist)", and are
c  used by other subroutines to initiate the variable.
c
90         continue
           jbeg = jend + 1
           call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                   , word32 )
           call nlsdac18 ( line, jbeg, word32 )
           call nlsdac19 ( word32, i1 )
           i2 = i1
           namelist(1,ilist)(9:12) = word32(1:4)
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           match = 1
           if (char1 .eq. ')') match = 2
           if (char1 .eq. ':') match = 3
           go to (80, 100, 110) match
100        continue
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           if (char1 .ne. '=') go to 80
           go to 130
110        continue
           jbeg = jend + 1
           call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                   , word32 )
           call nlsdac18 ( line, jbeg, word32 )
           namelist(1,ilist)(13:16) = word32(1:4)
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           match = 1
           if (char1 .eq. ')') match = 2
           if (char1 .eq. ',') match = 3
           go to (80, 100, 120) match
120        continue
           jbeg = jend + 1
           call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                   , word32 )
           call nlsdac18 ( line, jbeg, word32 )
           namelist(1,ilist)(17:20) = word32(1:4)
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           if (char1 .ne. ':') go to 80
           jbeg = jend + 1
           call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                   , word32 )
           call nlsdac18 ( line, jbeg, word32 )
           namelist(1,ilist)(21:24) = word32(1:4)
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           if (char1 .ne. ')') go to 80
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           if (char1 .ne. '=') go to 80
c
c      Read the character representation of the variable value.
c
130        continue
           jbeg    = jend + 1
           call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                   , word32 )
           jbegold = jbeg
           jendold = jend
           oldline = line
           jbeg    = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           if (char1 .eq. '*') then
             char1 = namelist(1,ilist)(32:32)
             if (char1 .ne. 'v') call nlsdac15 ( line, jbeg, 8 )
             if (i1 .eq. 0) i1 = 1
             call nlsdac18 ( line, jbeg, word32 )
             call nlsdac19 ( word32, i2 )
             i2 = i1 + i2 - 1
             call nlsdac20 ( i2, word32 )
             namelist(1,ilist)(13:16) = word32(1:4)
             call nlsdac20 ( i1, word32 )
             namelist(1,ilist)( 9:12) = word32(1:4)
             jbeg = jend + 1
             call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                     , word32 )
           else
             jbeg = jbegold
             jend = jendold
             if (line .ne. oldline) then
               backspace (nlunit)
               line = oldline
             endif
           endif
           ilisto = ilist
           char1  = namelist(1,ilist)(30:30)
           if (char1 .eq. 'l') call nlsdac22 ( line, jbeg, word32 )
           if (char1 .eq. 'i') call nlsdac18 ( line, jbeg, word32 )
           if (char1 .eq. 'r') call nlsdac23 ( line, jbeg, word32 )
           if (char1 .eq. 'c') then
c
c      Single or double quotes must surround each character string.  If
c  a second quote is not found, it will be assumed that the character
c  variable is being assigned a string longer than 30 characters.  In
c  this case, multiple entries are made in "namelist(1,ilist)" for the
c  character variable, with successive pieces of the assigned value
c  stored in "namelist(2,ilist)".  Continuation characters (&) are
c  appended to each piece of the input character value to tell the
c  subroutines which set the values of character variables that each
c  entry is to be concatenated, rather than overwriting the previous
c  entry.
c
             char1 = word32(1:1)
             if ( (char1 .ne. '"') .and. (char1 .ne. "'") )
     1         call nlsdac15 ( line, jbeg, 10 )
             temp32           = blank32
             temp32(1:jran-1) = word32(2:jran)
             word32           = temp32
             jran             = jran - 1
140          continue
             char1 = word32(jran:jran)
             if ( (char1 .ne. '"') .and. (char1 .ne. "'") ) then
               char1 = namelist(1,ilist)(32:32)
               if ( (char1 .eq. 'v') .and. (i1 .eq. 0) ) then
                 namelist(1,ilist)(9:9) = '1'
                 i1 = 1
               endif
               if (jran .le. 30) then
                 jbeg = - jend - 1
               endif
               if (jran .eq. 31) then
                 jran = 30
                 jbeg = - jend
               endif
               if (jran .eq. 32) then
                 jran = 30
                 jbeg = - jend + 1
               endif
               word32(31:31) = char(jran)
               word32(32:32) = '&'
               namelist(2,ilist) = word32
               word32 = namelist(1,ilist)
               nlist  = nlist + 1
               ilist  = nlist
               namelist(1,ilist) = word32
               call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                       , word32 )
               go to 140
             endif
             temp32           = blank32
             temp32(1:jran-1) = word32(1:jran-1)
             word32           = temp32
           endif
           namelist(2,ilist) = word32
           ilist             = ilisto
c
c      Make sure next character is a comma or a dollar sign.
c
           jbeg = jend + 1
           call nlsdac17 ( nlunit, line, jbeg, jend, char1 )
           if ( (char1 .ne. ',') .and. (char1 .ne. '$') ) go to 80
           jend = jbeg
c
c      If the variable is a vector, check to see if there are more data
c  (a propos for the syntax: iplot=1,1,1,2,0).
c
           if (namelist(1,ilist)(32:32) .eq. 'v') then
             if (i1 .eq. 0) then
               namelist(1,ilist)(9:9) = '1'
               i1 = 1
             endif
             if (i2 .eq. 0) i2 = i1
             if (char1 .eq. '$') return
             jbegold = jbeg
             jendold = jend
             oldline = line
             jbeg = jend + 1
             call nlsdac16 ( nlunit, line, jbeg, jend, jran, iqte
     1                     , word32 )
             char2 = word32(1:1)
             jbeg  = jend + 1
             call nlsdac17 ( nlunit, line, jbeg, jend, char3 )
             if ( (char3 .eq. ',') .or. (char3 .eq. '$') .or.
     1            (char3 .eq. '*') .or. (char2 .eq. '"') .or.
     2            (char2 .eq. "'") ) then
               word32 = namelist(1,ilist)
               nlist = nlist + 1
               ilist = nlist
               namelist(1,ilist)( 1: 8) = word32( 1: 8)
               namelist(1,ilist)(25:32) = word32(25:32)
               i1 = i2 + 1
               i2 = i1
               call nlsdac20 ( i1, word32 )
               namelist(1,ilist)( 9:12) = word32(1:4)
             endif
             jbeg = jbegold
             jend = jendold
             if (line .ne. oldline) then
               backspace (nlunit)
               line = oldline
             endif
             if ( (char3 .eq. ',') .or. (char3 .eq. '$') .or.
     1            (char3 .eq. '*') .or. (char2 .eq. '"') .or.
     2            (char2 .eq. "'") ) go to 130
           endif
c
c     Go on to next variable in "line".
c
           if (char1 .eq. '$') return
           go to 40
         endif
       endif
c
c      No match for "cname" in current "line".  Read next "line".
c
       go to 30
c
c      Error:  "cname" does not exist, or was out of order and thus
c  skipped over.
c
150    continue
       call nlsdac15 ( cname, 0, 6 )
c
c      Error:  Problem reading input deck.
c
160    continue
       call nlsdac15 ( blank32, 0, 7 )
c
c-----------------------------------------------------------------------
c----------------------- Read format statements ------------------------
c-----------------------------------------------------------------------
c
1010   format(a72)
c
c-----------------------------------------------------------------------
c---------------------- Write format statements ------------------------
c-----------------------------------------------------------------------
c
2010   format(a72)
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 2             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 3             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac03 ( cscalar, cstring, namelist, nmax )
c
c    dac:namelist.nlsdac03 <--------------------- fills character scalar
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1: February 1991, by David Clarke; rewritten to allow
c                  character strings to be specified in pieces.
c
c  PURPOSE:  This subroutine assigns the input value to a character
c  scalar.
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*32  blank32 , word32
       character*(*) cscalar
       integer       nmax    , i       , ibeg    , mult    , lbeg
     1             , lend    , lran
c
       character*32  namelist(   2,   *)
c
c      Data statements
c
       data          blank8   / '        ' /
       data          blank32  / '                                ' /
c
c-----------------------------------------------------------------------
c
       mult = 0
       ibeg = 1
       lbeg = 1
10     continue
c
c      Find next entry in "namelist(1,*)" which matches the name of the
c  variable to be set ("cstring").
c
       do 20 i=ibeg,nmax
         word8 = namelist(1,i)(1:8)
         if (word8 .eq. cstring) go to 30
         if (word8 .eq. blank8 ) return
20     continue
       return
30     continue
       word32 = namelist(2,i)
c
c      If character string is longer than 30 characters (final character
c  is '&'), it has been stored in "namelist" in pieces.  Thus, set the
c  next piece of "cscalar" to the contents of "namelist(2,i)", and
c  then go back to read the next piece.
c
       if (word32(32:32) .eq. '&') then
         mult = 1
         lran = ichar ( word32(31:31) )
         lend = lbeg + lran - 1
         cscalar(lbeg:lend) = word32(1:lran)
         lbeg = lend + 1
         ibeg = i + 1
         go to 10
       endif
c
c      If the character string was stored in pieces, "mult" will have
c  been set to 1.  If final character is not a '&', then this is the
c  last piece to read in.  Do not include trailing blanks in "cscalar".
c
c      If "mult" is 0, then the character string is stored in its
c  entirety in "namelist(2,i)".  Thus, set cscalar trivially.
c
       if (mult .eq. 1) then
         do 60 lran=31,1,-1
           if (word32(lran:lran) .ne. ' ') go to 70
60       continue
70       continue
         lend = lbeg + lran - 1
         cscalar(lbeg:lend) = word32(1:lran)
       else
         if (word32 .ne. blank32) cscalar = word32
       endif
c
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 3             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 4             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac04 ( iscalar, cstring, namelist, nmax )
c
c    dac:namelist.nlsdac04 <----------------------- fills integer scalar
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input value to an integer
c  scalar.
c
c  EXTERNALS:  NLSDAC19
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*32  blank32 , word32
       integer       iscalar , nmax    , i
c
       character*32  namelist(   2,   *)
c
c      Data statements
c
       data          blank8   / '        ' /
       data          blank32  / '                                ' /
c
c      External statements
c
       external      nlsdac19
c
c-----------------------------------------------------------------------
c
       do 10 i=1,nmax
         word8 = namelist(1,i)(1:8)
         if (word8 .eq. cstring) go to 20
         if (word8 .eq. blank8 ) return
10     continue
       return
20     continue
       word32 = namelist(2,i)
       if (word32 .ne. blank32) call nlsdac19 ( word32, iscalar )
c
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 4             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 5             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac05 ( rscalar, cstring, namelist, nmax )
c
c    dac:namelist.nlsdac05 <-------------------------- fills real scalar
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input value to a real scalar.
c
c  EXTERNALS:  NLSDAC21
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*32  blank32 , word32
       integer       nmax    , i
       real          rscalar
c
       character*32  namelist(   2,   *)
c
c      Data statements
c
       data          blank8   / '        ' /
       data          blank32  / '                                ' /
c
c      External statements
c
       external      nlsdac21
c
c-----------------------------------------------------------------------
c
       do 10 i=1,nmax
         word8 = namelist(1,i)(1:8)
         if (word8 .eq. cstring) go to 20
         if (word8 .eq. blank8 ) return
10     continue
       return
20     continue
       word32 = namelist(2,i)
       if (word32 .ne. blank32) call nlsdac21 ( word32, rscalar )
c
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 5             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 6             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac06 ( lscalar, cstring, namelist, nmax )
c
c    dac:namelist.nlsdac06 <----------------------- fills logical scalar
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input value to a logical
c  scalar.
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       logical       lscalar
       character*8   cstring , word8   , blank8
       integer       nmax    , i
c
       character*32  namelist(   2,   *)
c
c      Data statements
c
       data          blank8   / '        ' /
c
c-----------------------------------------------------------------------
c
       do 10 i=1,nmax
         word8 = namelist(1,i)(1:8)
         if (word8 .eq. cstring) go to 20
         if (word8 .eq. blank8 ) return
10     continue
       return
20     continue
       word8 = namelist(2,i)(1:8)
       if (word8 .eq. '.true.  ') lscalar = .true.
       if (word8 .eq. '.t.     ') lscalar = .true.
       if (word8 .eq. '.false. ') lscalar = .false.
       if (word8 .eq. '.f.     ') lscalar = .false.
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 6             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 7             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac07 ( cvector, cstring, namelist, nmax )
c
c    dac:namelist.nlsdac07 <--------------------- fills character vector
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1: February 1991, by David Clarke; rewritten to allow
c                  character strings to be specified in pieces.
c
c  PURPOSE:  This subroutine assigns the input values to a character
c  vector.
c
c  EXTERNALS:  NLSDAC19
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*12  cstr12  , word12
       character*32  word32  , blank32
       integer       nmax    , mult    , i       , ibeg    , j
     1             , jbeg    , jend    , lbeg    , lend    , lran
c
       character*(*) cvector(*)
c
       character*32  namelist(   2,   *)
c
c      Data statements
c
       data          blank8   / '        ' /
       data          blank32  / '                                ' /
c
c      External statements
c
       external      nlsdac19
c
c-----------------------------------------------------------------------
c
       ibeg  = 1
10     continue
       lbeg  = 1
       mult  = 0
c
c      Find next entry in "namelist(1,*)" which matches the name of the
c  variable to be set ("cstring").
c
       do 20 i=ibeg,nmax
         word8 = namelist(1,i)(1:8)
         if (word8 .eq. cstring) go to 30
         if (word8 .eq. blank8 ) return
20     continue
30     continue
       cstr12 = namelist(1,i)(1:12)
c
c      Set range of variable index (jbeg through jend) to be set to
c  what will be stored in "cvector(jbeg)".
c
       word32 = blank32
       word32(1:4) = namelist(1,i)(9:12)
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = namelist(1,i)(13:16)
       if (word32 .eq. blank32) then
         jend = jbeg
       else
         call nlsdac19 ( word32, jend )
       endif
c
c      If character string is longer than 30 characters (final character
c  is '&'), it has been stored in "namelist" in pieces.  Thus, set the
c  next piece of "cvector(jbeg)" to the contents of "namelist(2,i)", and
c  then read the next piece.
c
40     continue
       word32 = namelist(2,i)
       if (word32(32:32) .eq. '&') then
         mult = 1
         lran = ichar ( word32(31:31) )
         lend = lbeg + lran - 1
         cvector(jbeg)(lbeg:lend) = word32(1:lran)
         lbeg = lend + 1
         ibeg = i + 1
         do 70 i=ibeg,nmax
           word12 = namelist(1,i)(1:12)
           if (word12 .eq. cstr12 ) go to 80
70       continue
80       continue
         go to 40
       endif
c
c      If the character string was stored in pieces, "mult" will have
c  been set to 1.  If final character is not a '&', then this is the
c  last piece to read in.  Do not include trailing blanks in
c  "cvector(jbeg)".
c
c      If "mult" is 0, then the character string is stored in its
c  entirety in "namelist(2,i)".  Thus, set cvector(jbeg) trivially.
c
       if (mult .eq. 1) then
         do 90 lran=31,1,-1
           if (word32(lran:lran) .ne. ' ') go to 100
90       continue
100      continue
         lend = lbeg + lran - 1
         cvector(jbeg)(lbeg:lend) = word32(1:lran)
       else
         if (word32 .ne. blank32) cvector(jbeg) = word32
       endif
c
c      Set desired range of "cvector" to the desired constant.
c
       do 110 j=jbeg,jend
         cvector(j) = cvector(jbeg)
110    continue
c
c      Search for next range of elements in "cvector" to be set.
c
       ibeg = i + 1
       if (ibeg .gt. nmax) return
       go to 10
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 7             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 8             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac08 ( ivector, cstring, namelist, nmax )
c
c    dac:namelist.nlsdac08 <----------------------- fills integer vector
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input values to an integer
c  vector.
c
c  EXTERNALS:  NLSDAC19
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*32  word32  , blank32
       integer       nmax    , i       , ibeg    , j       , jbeg
     1             , jend    , ival
c
       integer       ivector (   *)
c
       character*32  namelist(   2,   *)
c
c      Data statements
c
       data          blank8  / '        ' /
       data          blank32 / '                                ' /
c
c      External statements
c
       external      nlsdac19
c
c-----------------------------------------------------------------------
c
       ibeg = 1
10     continue
       do 20 i=ibeg,nmax
         word8 = namelist(1,i)(1:8)
         if (word8 .eq. blank8) return
         if (word8 .eq. cstring) go to 30
20     continue
30     continue
       word32 = namelist(2,i)
       if (word32 .eq. blank32) return
       ibeg = i + 1
       if (ibeg .gt. nmax+1) return
       word32 = blank32
       word32(1:4) = namelist(1,i)(9:12)
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = namelist(1,i)(13:16)
       if (word32 .eq. blank32) then
         jend = jbeg
       else
         call nlsdac19 ( word32, jend )
       endif
       call nlsdac19 ( namelist(2,i), ival )
       do 40 j=jbeg,jend
         ivector(j) = ival
40     continue
       go to 10
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 8             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 0 9             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac09 ( rvector, cstring, namelist, nmax )
c
c    dac:namelist.nlsdac09 <-------------------------- fills real vector
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input values to a real vector.
c
c  EXTERNALS:  NLSDAC19, NLSDAC21
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*32  word32  , blank32
       integer       nmax    , i       , ibeg    , j       , jbeg
     1             , jend
       real          rval
c
       real          rvector (   *)
c
       character*32  namelist(   2,   *)
c
c      Data statements
c
       data          blank8  / '        ' /
       data          blank32 / '                                ' /
c
c      External statements
c
       external      nlsdac19, nlsdac21
c
c-----------------------------------------------------------------------
c
       ibeg = 1
10     continue
       do 20 i=ibeg,nmax
         word8 = namelist(1,i)(1:8)
         if (word8 .eq. blank8) return
         if (word8 .eq. cstring) go to 30
20     continue
30     continue
       word32 = namelist(2,i)
       if (word32 .eq. blank32) return
       ibeg = i + 1
       if (ibeg .gt. nmax+1) return
       word32 = blank32
       word32(1:4) = namelist(1,i)(9:12)
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = namelist(1,i)(13:16)
       if (word32 .eq. blank32) then
         jend = jbeg
       else
         call nlsdac19 ( word32, jend )
       endif
       call nlsdac21 ( namelist(2,i), rval )
       do 40 j=jbeg,jend
         rvector(j) = rval
40     continue
       go to 10
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 0 9             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 1 0             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac10 ( lvector, cstring, namelist, nmax )
c
c    dac:namelist.nlsdac10 <----------------------- fills logical vector
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input values to a logical
c  vector.
c
c  EXTERNALS:  NLSDAC19
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       logical       lval
       character*8   cstring , word8   , blank8
       character*32  word32  , blank32
       integer       nmax    , i       , ibeg    , j       , jbeg
     1             , jend
c
       logical       lvector (   *)
c
       character*32  namelist(   2,   *)
c
c      Data statements
c
       data          blank8  / '        ' /
       data          blank32 / '                                ' /
c
c      External statements
c
       external      nlsdac19
c
c-----------------------------------------------------------------------
c
       ibeg = 1
10     continue
       do 20 i=ibeg,nmax
         word8 = namelist(1,i)(1:8)
         if (word8 .eq. blank8) return
         if (word8 .eq. cstring) go to 30
20     continue
30     continue
       word32 = namelist(2,i)
       if (word32 .eq. blank32) return
       ibeg = i + 1
       if (ibeg .gt. nmax+1) return
       word32 = blank32
       word32(1:4) = namelist(1,i)(9:12)
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = namelist(1,i)(13:16)
       if (word32 .eq. blank32) then
         jend = jbeg
       else
         call nlsdac19 ( word32, jend )
       endif
       word8 = namelist(2,i)(1:8)
       if (word8 .eq. '.true.  ') lval = .true.
       if (word8 .eq. '.t.     ') lval = .true.
       if (word8 .eq. '.false. ') lval = .false.
       if (word8 .eq. '.f.     ') lval = .false.
       do 40 j=jbeg,jend
         lvector(j) = lval
40     continue
       go to 10
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 1 0             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 1 1             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac11 ( carray, cstring, namelist, nmax, nx )
c
c    dac:namelist.nlsdac11 <------ fills two-dimensional character array
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1: February 1991, by David Clarke; rewritten to allow
c                  character strings to be specified in pieces.
c
c  PURPOSE:  This subroutine assigns the input values to a two-
c  dimensional character array.
c
c  EXTERNALS:  NLSDAC19
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*24  cstr24  , word24
       character*32  word32  , blank32
       integer       nmax    , nx      , mult    , i       , ibeg
     1             , j       , jbeg    , jend    , k       , kbeg
     2             , kend    , lbeg    , lend    , lran
c
       character*32  namelist(   2,   *)
       character*(*) carray  (  nx,   *)
c
c      Data statements
c
       data          blank8   / '        ' /
       data          blank32  / '                                ' /
c
c      External statements
c
       external      nlsdac19
c
c-----------------------------------------------------------------------
c
       ibeg  = 1
10     continue
       lbeg  = 1
       mult  = 0
c
c      Find next entry in "namelist(1,*)" which matches the name of the
c  variable to be set ("cstring").
c
       do 20 i=ibeg,nmax
         word8 = namelist(1,i)(1:8)
         if (word8 .eq. cstring) go to 30
         if (word8 .eq. blank8 ) return
20     continue
30     continue
       cstr24 = namelist(1,i)(1:24)
c
c      Set range of variable index (jbeg through jend; kbeg through
c  kend) to be set to what will be stored in "carray(jbeg,kbeg)".
c
       word32 = blank32
       word32(1:4) = namelist(1,i)(9:12)
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = namelist(1,i)(13:16)
       call nlsdac19 ( word32, jend )
       word32(1:4) = namelist(1,i)(17:20)
       call nlsdac19 ( word32, kbeg )
       word32(1:4) = namelist(1,i)(21:24)
       call nlsdac19 ( word32, kend )
c
c      If character string is longer than 30 characters (final character
c  is '&'), it has been stored in "namelist" in pieces.  Thus, set the
c  next piece of "carray(jbeg,kbeg)" to the contents of "namelist(2,i)",
c  and then read the next piece.
c
40     continue
       word32 = namelist(2,i)
       if (word32(32:32) .eq. '&') then
         mult = 1
         lran = ichar ( word32(31:31) )
         lend = lbeg + lran - 1
         carray(jbeg,kbeg)(lbeg:lend) = word32(1:lran)
         lbeg = lend + 1
         ibeg = i + 1
         do 70 i=ibeg,nmax
           word24 = namelist(1,i)(1:24)
           if (word24 .eq. cstr24 ) go to 80
70       continue
80       continue
         go to 40
       endif
c
c      If the character string was stored in pieces, "mult" will have
c  been set to 1.  If final character is not a '&', then this is the
c  last piece to read in.  Do not include trailing blanks in
c  "carray(jbeg,kbeg)".
c
c      If "mult" is 0, then the character string is stored in its
c  entirety in "namelist(2,i)".  Thus, set carray(jbeg,kbeg) trivially.
c
       if (mult .eq. 1) then
         do 90 lran=31,1,-1
           if (word32(lran:lran) .ne. ' ') go to 100
90       continue
100      continue
         lend = lbeg + lran - 1
         carray(jbeg,kbeg)(lbeg:lend) = word32(1:lran)
       else
         if (word32 .ne. blank32) carray(jbeg,kbeg) = word32
       endif
c
c      Set desired range of "carray" to the desired constant.
c
       do 120 k=kbeg,kend
         do 110 j=jbeg,jend
           carray(j,k) = carray(jbeg,kbeg)
110      continue
120    continue
c
c      Search for next range of elements in "carray" to be set.
c
       ibeg = i + 1
       if (ibeg .gt. nmax) return
       go to 10
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 1 1             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 1 2             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac12 ( iarray, cstring, namelist, nmax, nx )
c
c    dac:namelist.nlsdac12 <-------- fills two-dimensional integer array
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input values to a two-
c  dimensional integer array.
c
c  EXTERNALS:  NLSDAC19
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*32  word32  , blank32
       integer       nmax    , nx      , i       , ibeg    , j
     1             , jbeg    , jend    , k       , kbeg    , kend
     2             , ival
c
       character*32  namelist(   2,   *)
       integer       iarray  (  nx,   *)
c
c      Data statements
c
       data          blank8  / '        ' /
       data          blank32 / '                                ' /
c
c      External statements
c
       external      nlsdac19
c
c-----------------------------------------------------------------------
c
       ibeg = 1
10     continue
       do 20 i=ibeg,nmax
         word8 = namelist(1,i)(1:8)
         if (word8 .eq. blank8) return
         if (word8 .eq. cstring) go to 30
20     continue
30     continue
       word32 = namelist(2,i)
       if (word32 .eq. blank32) return
       ibeg = i + 1
       if (ibeg .gt. nmax+1) return
       word32 = blank32
       word32(1:4) = namelist(1,i)(9:12)
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = namelist(1,i)(13:16)
       call nlsdac19 (word32, jend )
       word32(1:4) = n amelist(1,i)(17:20)
       call nlsdac19 ( word32, kbeg )
       word32(1:4) = namelist(1,i)(21:24)
       call nlsdac19 ( word32, kend )
       call nlsdac19 ( namelist(2,i), ival )
       do 50 k=kbeg,kend
         do 40 j=jbeg,jend
           iarray(j,k) = ival
40       continue
50     continue
       go to 10
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 1 2             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 1 3             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac13 ( rarray, cstring, namelist, nmax, nx )
c
c    dac:namelist.nlsdac13 <----------- fills two-dimensional real array
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input values to a two-
c  dimensional real array.
c
c  EXTERNALS:  NLSDAC19, NLSDAC21
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*8   cstring , word8   , blank8
       character*32  word32  , blank32
       integer       nmax    , nx      , i       , ibeg    , j
     1             , jbeg    , jend    , k       , kbeg    , kend
       real          rval
c
       character*32  namelist(   2,   *)
       real          rarray  (  nx,   *)
c
c      Data statements
c
       data          blank8  / '        ' /
       data          blank32 / '                                ' /
c
c      External statements
c
       external      nlsdac19, nlsdac21
c
c-----------------------------------------------------------------------
c
       ibeg = 1
10     continue
       do 20 i=ibeg,nmax
         word8 = namelist(1,i)(1:8)
         if (word8 .eq. blank8) return
         if (word8 .eq. cstring) go to 30
20     continue
30     continue
       word32 = namelist(2,i)
       if (word32 .eq. blank32) return
       ibeg = i + 1
       if (ibeg .gt. nmax+1) return
       word32 = blank32
       word32(1:4) = namelist(1,i)(9:12)
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = namelist(1,i)(13:16)
       call nlsdac19 ( word32, jend )
       word32(1:4) = namelist(1,i)(17:20)
       call nlsdac19 ( word32, kbeg )
       word32(1:4) = namelist(1,i)(21:24)
       call nlsdac19 ( word32, kend )
       call nlsdac21 ( namelist(2,i), rval )
       do 50 k=kbeg,kend
         do 40 j=jbeg,jend
           rarray(j,k) = rval
40       continue
50     continue
       go to 10
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 1 3             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 1 4             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac14 ( larray, cstring, namelist, nmax, nx )
c
c    dac:namelist.nlsdac14 <-------- fills two-dimensional logical array
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine assigns the input values to a two-
c  dimensional logical array.
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       logical       lval
       character*8   cstring , word8   , blank8
       character*32  word32  , blank32
       integer       nmax    , nx      , i       , ibeg    , j
     1             , jbeg    , jend    , k       , kbeg    , kend
c
       logical       larray  (  nx,   *)
       character*32  namelist(   2,   *)
c
c      Data statements
c
       data          blank8  / '        ' /
       data          blank32 / '                                ' /
c
c-----------------------------------------------------------------------
c
       ibeg = 1
10     continue
       do 20 i=ibeg,nmax
         word8 = namelist(1,i)(1:8)
         if (word8 .eq. blank8) return
         if (word8 .eq. cstring) go to 30
20     continue
30     continue
       word32 = namelist(2,i)
       if (word32 .eq. blank32) return
       ibeg = i + 1
       if (ibeg .gt. nmax+1) return
       word32 = blank32
       word32(1:4) = namelist(1,i)(9:12)
       call nlsdac19 ( word32, jbeg )
       word32(1:4) = namelist(1,i)(13:16)
       call nlsdac19 ( word32, jend )
       word32(1:4) = namelist(1,i)(17:20)
       call nlsdac19 ( word32, kbeg )
       word32(1:4) = namelist(1,i)(21:24)
       call nlsdac19 ( word32, kend )
       word8 = namelist(2,i)(1:8)
       if (word8 .eq. '.true.  ') lval = .true.
       if (word8 .eq. '.t.     ') lval = .true.
       if (word8 .eq. '.false. ') lval = .false.
       if (word8 .eq. '.f.     ') lval = .false.
       do 50 k=kbeg,kend
         do 40 j=jbeg,jend
           larray(j,k) = lval
40       continue
50     continue
       go to 10
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 1 4             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 1 5             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac15 ( phrase, ipoint, ierror )
c
c    dac:namelist.nlsdac15 <---------------------- issues error messages
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine is called when an error is detected in the
c  namelist by one of the subroutines.  The appropriate error message
c  is issued to the CRT, and execution is terminated.
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*(*) phrase
       integer       ipoint  , ierror
c
c-----------------------------------------------------------------------
c
       write (6, 2010)
       write (6, 2020) phrase
c
       if (ipoint .eq.  1) write (6,2030)
       if (ipoint .eq.  2) write (6,2040)
       if (ipoint .eq.  3) write (6,2050)
       if (ipoint .eq.  4) write (6,2060)
       if (ipoint .eq.  5) write (6,2070)
       if (ipoint .eq.  6) write (6,2080)
       if (ipoint .eq.  7) write (6,2090)
       if (ipoint .eq.  8) write (6,2100)
       if (ipoint .eq.  9) write (6,2110)
       if (ipoint .eq. 10) write (6,2120)
       if (ipoint .eq. 11) write (6,2130)
       if (ipoint .eq. 12) write (6,2140)
       if (ipoint .eq. 13) write (6,2150)
       if (ipoint .eq. 14) write (6,2160)
       if (ipoint .eq. 15) write (6,2170)
       if (ipoint .eq. 16) write (6,2180)
       if (ipoint .eq. 17) write (6,2190)
       if (ipoint .eq. 18) write (6,2200)
       if (ipoint .eq. 19) write (6,2210)
       if (ipoint .eq. 20) write (6,2220)
       if (ipoint .eq. 21) write (6,2230)
       if (ipoint .eq. 22) write (6,2240)
       if (ipoint .eq. 23) write (6,2250)
       if (ipoint .eq. 24) write (6,2260)
       if (ipoint .eq. 25) write (6,2270)
       if (ipoint .eq. 26) write (6,2280)
       if (ipoint .eq. 27) write (6,2290)
       if (ipoint .eq. 28) write (6,2300)
       if (ipoint .eq. 29) write (6,2310)
       if (ipoint .eq. 30) write (6,2320)
       if (ipoint .eq. 31) write (6,2330)
       if (ipoint .eq. 32) write (6,2340)
       if (ipoint .eq. 33) write (6,2350)
       if (ipoint .eq. 34) write (6,2360)
       if (ipoint .eq. 35) write (6,2370)
       if (ipoint .eq. 36) write (6,2380)
       if (ipoint .eq. 37) write (6,2390)
       if (ipoint .eq. 38) write (6,2400)
       if (ipoint .eq. 39) write (6,2410)
       if (ipoint .eq. 40) write (6,2420)
       if (ipoint .eq. 41) write (6,2430)
       if (ipoint .eq. 42) write (6,2440)
       if (ipoint .eq. 43) write (6,2450)
       if (ipoint .eq. 44) write (6,2460)
       if (ipoint .eq. 45) write (6,2470)
       if (ipoint .eq. 46) write (6,2480)
       if (ipoint .eq. 47) write (6,2490)
       if (ipoint .eq. 48) write (6,2500)
       if (ipoint .eq. 49) write (6,2510)
       if (ipoint .eq. 50) write (6,2520)
       if (ipoint .eq. 51) write (6,2530)
       if (ipoint .eq. 52) write (6,2540)
       if (ipoint .eq. 53) write (6,2550)
       if (ipoint .eq. 54) write (6,2560)
       if (ipoint .eq. 55) write (6,2570)
       if (ipoint .eq. 56) write (6,2580)
       if (ipoint .eq. 57) write (6,2590)
       if (ipoint .eq. 58) write (6,2600)
       if (ipoint .eq. 59) write (6,2610)
       if (ipoint .eq. 60) write (6,2620)
       if (ipoint .eq. 61) write (6,2630)
       if (ipoint .eq. 62) write (6,2640)
       if (ipoint .eq. 63) write (6,2650)
       if (ipoint .eq. 64) write (6,2660)
       if (ipoint .eq. 65) write (6,2670)
       if (ipoint .eq. 66) write (6,2680)
       if (ipoint .eq. 67) write (6,2690)
       if (ipoint .eq. 68) write (6,2700)
       if (ipoint .eq. 69) write (6,2710)
       if (ipoint .eq. 70) write (6,2720)
       if (ipoint .eq. 71) write (6,2730)
       if (ipoint .eq. 72) write (6,2740)
c
       if (ierror .eq.  1) write (6,2750)
       if (ierror .eq.  2) write (6,2760)
       if (ierror .eq.  3) write (6,2770)
       if (ierror .eq.  4) write (6,2780)
       if (ierror .eq.  5) write (6,2790)
       if (ierror .eq.  6) write (6,2800)
       if (ierror .eq.  7) write (6,2810)
       if (ierror .eq.  8) write (6,2820)
       if (ierror .eq.  9) write (6,2830)
       if (ierror .eq. 10) write (6,2840)
       if (ierror .eq. 11) write (6,2850)
       if (ierror .eq. 12) write (6,2860)
       if (ierror .eq. 13) write (6,2870)
       if (ierror .eq. 14) write (6,2880)
       if (ierror .eq. 15) write (6,2890)
       if (ierror .eq. 16) write (6,2900)
       if (ierror .eq. 17) write (6,2910)
       if (ierror .eq. 18) write (6,2920)
       if (ierror .eq. 19) write (6,2930)
       if (ierror .eq. 20) write (6,2940)
c
       write (6,2010)
       stop
c
c-----------------------------------------------------------------------
c---------------------- Write format statements ------------------------
c-----------------------------------------------------------------------
c
2010   format('************************************'
     1       ,'************************************')
2020   format( (a) )
2030   format(    '^')
2040   format( 1x,'^')
2050   format( 2x,'^')
2060   format( 3x,'^')
2070   format( 4x,'^')
2080   format( 5x,'^')
2090   format( 6x,'^')
2100   format( 7x,'^')
2110   format( 8x,'^')
2120   format( 9x,'^')
2130   format(10x,'^')
2140   format(11x,'^')
2150   format(12x,'^')
2160   format(13x,'^')
2170   format(14x,'^')
2180   format(15x,'^')
2190   format(16x,'^')
2200   format(17x,'^')
2210   format(18x,'^')
2220   format(19x,'^')
2230   format(20x,'^')
2240   format(21x,'^')
2250   format(22x,'^')
2260   format(23x,'^')
2270   format(24x,'^')
2280   format(25x,'^')
2290   format(26x,'^')
2300   format(27x,'^')
2310   format(28x,'^')
2320   format(29x,'^')
2330   format(30x,'^')
2340   format(31x,'^')
2350   format(32x,'^')
2360   format(33x,'^')
2370   format(34x,'^')
2380   format(35x,'^')
2390   format(36x,'^')
2400   format(37x,'^')
2410   format(38x,'^')
2420   format(39x,'^')
2430   format(40x,'^')
2440   format(41x,'^')
2450   format(42x,'^')
2460   format(43x,'^')
2470   format(44x,'^')
2480   format(45x,'^')
2490   format(46x,'^')
2500   format(47x,'^')
2510   format(48x,'^')
2520   format(49x,'^')
2530   format(50x,'^')
2540   format(51x,'^')
2550   format(52x,'^')
2560   format(53x,'^')
2570   format(54x,'^')
2580   format(55x,'^')
2590   format(56x,'^')
2600   format(57x,'^')
2610   format(58x,'^')
2620   format(59x,'^')
2630   format(60x,'^')
2640   format(61x,'^')
2650   format(62x,'^')
2660   format(63x,'^')
2670   format(64x,'^')
2680   format(65x,'^')
2690   format(66x,'^')
2700   format(67x,'^')
2710   format(68x,'^')
2720   format(69x,'^')
2730   format(70x,'^')
2740   format(71x,'^')
2750   format('NAMELIST ERROR ---> column 1 reserved for comment '
     1       ,'sentinal: c')
2760   format('NAMELIST ERROR ---> column 2 reserved for $ delimeter')
2770   format('NAMELIST ERROR ---> variable not found in namelist')
2780   format('NAMELIST ERROR ---> unexpected character - check syntax')
2790   format('NAMELIST ERROR ---> invalid logical expression')
2800   format('NAMELIST ERROR ---> namelist does not exist, or is out '
     1       ,'of sequence')
2810   format('NAMELIST ERROR ---> error reading input deck')
2820   format('NAMELIST ERROR ---> variable not declared as a vector')
2830   format('NAMELIST ERROR ---> next namelist begun before closing '
     1       ,'quote found')
2840   format('NAMELIST ERROR ---> missing opening quote')
2850   format('NAMELIST ERROR ---> blank data')
2860   format('NAMELIST ERROR ---> premature end of file')
2870   format('NAMELIST ERROR ---> missing $ sentinal to close '
     1       ,'namelist')
2880   format('NAMELIST ERROR ---> exponent must not exceed 999')
2890   format('NAMELIST ERROR ---> no more than 15 digits in single '
     1       ,'precision')
2900   format('NAMELIST ERROR ---> ')
2910   format('NAMELIST ERROR ---> ')
2920   format('NAMELIST ERROR ---> ')
2930   format('NAMELIST ERROR ---> ')
2940   format('NAMELIST ERROR ---> ')
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 1 5             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////              N L S D A C 1 6              \\\\\\\\\\
c
c=======================================================================
c
c
       subroutine nlsdac16 ( nlunit, line, jbeg, jend, jran, iquote
     1                     , word32 )
c
c    dac:namelist.nlsdac16 <-------------- returns next "word" in "line"
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine returns the next 32 character "word" found
c  after the input position of "jbeg" in "line".  The beginning and
c  ending positions of "word" are returned as well ("jbeg" and "jend").
c
c  There are two types of delimiters which define "word".  The first
c  type include the double and single quotation marks [" and '].  The
c  quotes are returned with "word".  Both the beginning and ending of
c  "word" must be delimited with the same type of quotation mark.
c
c  The second type include the equals sign [=], the asterisk [*], the
c  comma [,], the full colon [:], the opening parenthesis [(], the
c  closing parenthseis [)], the dollar sign [$], the blank, and the end
c  of "line".  These delimiters are not returned with "word", and so
c  "word" may be as long as 32 characters.  "word" may begin and end
c  with different delimiters of the second type.  (Note that $ and the
c  end of "line" can only delimit the end of "word".)  Delimiters of the
c  first type may be used to end "word" beginning with a delimiter of
c  the first type, and will not be returned with "word".  The reverse is
c  not true.  Delimiters of the second type found between like quotation
c  marks are returned as characters in "word".  Note that the search for
c  the next word will start directly at abs(jbeg) if "jbeg" < 0.
c
c  If the end of "line" is encountered before the next "word" is found,
c  the next "line" is read.  Both "line" and "word" are returned to the
c  calling subroutine.
c
c  EXTERNALS:  NLSDAC15
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*1   char1   , char2   , char3
       character*32  word32  , blank32
       character*72  line    , oldline
       integer       nlunit  , jbeg    , jend    , jran    , iquote
     1             , j       , j1      , j2      , iflag
c
c      Data statements
c
       data          blank32 /'                                '/
c
c      External statements
c
       external      nlsdac15
c
c-----------------------------------------------------------------------
c
c      Scan "line" for beginning of next "word".
c
       iflag = 1
       if (jbeg .lt. 0) then
         iflag = - 1
         jbeg  = - jbeg
         if (jbeg .gt. 72) go to 30
         go to 70
       endif
       j1 = jbeg
       if (jbeg .eq. 0) j1 =  3
10     continue
       do 20 j=j1,72
         char1 = line(j:j)
         if ( (char1 .eq. ' ') .or. (char1 .eq. ',') .or.
     1        (char1 .eq. '=') .or. (char1 .eq. ')') .or.
     2        (char1 .eq. '(') .or. (char1 .eq. '*') .or.
     3        (char1 .eq. ':') ) go to 20
         jbeg = j
         go to 70
20     continue
30     continue
c
c      The only way for execution to have reached this point is for the
c  end of the line to have been reached without finding another "word".
c  Read next "line", and resume search.
c
       oldline = line
40     continue
       read  (nlunit, 1010, end=50, err=60) line
c
c      Error message if column 1 contains anything but a 'c', 'C', or a
c  blank.
c
       char2 = line(1:1)
       if ( (char2 .ne. 'c') .and. (char2 .ne. 'C') .and.
     1      (char2 .ne. ' ') ) call nlsdac15 ( line, 1, 1 )
c
c      Error message if column 2 contains anything but a '$' or a blank
c  (unless column 1 contains a 'c' or a 'C').
c
       char3 = line(2:2)
       if ( (char2 .ne. 'c') .and. (char2 .ne. 'C') .and.
     1      (char3 .ne. '$') .and. (char3 .ne. ' ') )
     2   call nlsdac15 ( line, 2, 2 )
c
c      If column 1 contains a 'c' or a 'C', the line is "commented out",
c  and is echoed on the terminal.
c
       if ( (char2 .eq. 'c') .or. (char2 .eq. 'C') ) then
         write (6, 2010) line
         go to 40
       endif
c
c      If column 2 contains a '$' sentinal, then a new namelist has
c  begun without the previous one being closed.  Issue error message.
c
       if (char3 .eq. '$') then
         if (iquote .eq. 0) then
           call nlsdac15 ( oldline, 72, 13 )
         else
           call nlsdac15 ( line, 2, 9 )
         endif
       endif
c
c      New "line" is OK, resume scanning.
c
       if (iflag .eq. -1) then
         jbeg = 3
         go to 70
       else
         j1 = 3
         go to 10
       endif
c
c      Error: premature end of file.
c
50     continue
       call nlsdac15 ( line, 72, 12 )
c
c      Error: problem reading input deck
c
60     continue
       call nlsdac15 ( blank32, 0, 7 )
c
c      "jbeg" has been determined.  Now find "jend", which cannot be
c  greater than "jbeg" + 31.
c
70     continue
       j1   = jbeg + 1
       j2   = jbeg + 31
       if (j2 .gt. 72) j2 = 72
       jend = jbeg
c
c      Case:  first delimiter is a single quote.
c
       if ( (iquote .eq. 1) .or. (char1 .eq. "'") ) then
         do 80 j=j1,j2
           jend  = j
           char1 = line(j:j)
           if (char1 .eq. "'") go to 110
80       continue
c
c      Closing single quote not found.  Set "iquote" to 1.
c
         iquote = 1
         go to 120
       endif
c
c      Case:  first delimiter is a double quote.
c
       if ( (iquote .eq. 2) .or. (char1 .eq. '"') ) then
         do 90 j=j1,j2
           jend  = j
           char1 = line(j:j)
           if (char1 .eq. '"') go to 110
90       continue
c
c      Closing double quote not found.  Set "iquote" to 2.
c
         iquote = 2
         go to 120
       endif
c
c      Case:  first delimiter is not a quote.
c
       do 100 j=j1,j2
         char1 = line(j:j)
         if ( (char1 .eq. ' ') .or. (char1 .eq. ',') .or.
     1        (char1 .eq. '=') .or. (char1 .eq. ')') .or.
     2        (char1 .eq. '(') .or. (char1 .eq. '*') .or.
     3        (char1 .eq. ':') .or. (char1 .eq. '"') .or.
     4        (char1 .eq. "'") .or. (char1 .eq. '$') ) go to 110
         jend = j
100    continue
110    continue
       iquote = 0
120    continue
       jran   = jend - jbeg + 1
       word32 = blank32
       word32(1:jran) = line(jbeg:jend)
c
       return
c
c-----------------------------------------------------------------------
c----------------------- Read format statements ------------------------
c-----------------------------------------------------------------------
c
1010   format(a72)
c
c-----------------------------------------------------------------------
c---------------------- Write format statements ------------------------
c-----------------------------------------------------------------------
c
2010   format(a72)
c
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////              N L S D A C 1 6              \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////              N L S D A C 1 7              \\\\\\\\\\
c
c=======================================================================
c
c
       subroutine nlsdac17 ( nlunit, line, jbeg, jend, char1 )
c
c    dac:namelist.nlsdac17 <----------- returns next non-blank character
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine returns the first non-blank character found
c  at or after the input position "jbeg" in "line".  If no non-blank
c  character is found, the next line is read and scanned for the first
c  non-blank character.
c
c  EXTERNALS:  NLSDAC15
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*1   char1   , char2   , char3
       character*8   blank8
       character*72  line    , oldline
       integer       nlunit  , jbeg    , jend    , j       , j1
c
c      Data statements
c
       data          blank8 / '        ' /
c
c      External statements
c
       external      nlsdac15
c
c-----------------------------------------------------------------------
c
c      Scan "line" for next non-blank character.
c
       j1 = jbeg
       if (j1 .eq. 0) j1 = 3
10     continue
       do 20 j=j1,72
         jbeg  = j
         jend  = j
         char1 = line(j:j)
         if (char1 .ne. ' ') return
20     continue
c
c      End of line reached before non-blank character found.  Read
c  next line.
c
       oldline = line
30     continue
       read  (nlunit, 1010, end=40, err=50) line
c
c      Error message if column 1 contains anything but a 'c', 'C', or a
c  blank.
c
       char2 = line(1:1)
       if ( (char2 .ne. 'c') .and. (char2 .ne. 'C') .and.
     1      (char2 .ne. ' ') ) call nlsdac15 ( line, 1, 1 )
c
c      Error message if column 2 contains anything but a '$' or a blank
c  (unless column 1 contains a 'c' or a 'C').
c
       char3 = line(2:2)
       if ( (char2 .ne. 'c') .and. (char2 .ne. 'C') .and.
     1      (char3 .ne. '$') .and. (char3 .ne. ' ') )
     2   call nlsdac15 ( line, 2, 2 )
c
c      If column 1 contains a 'c' or a 'C', the line is "commented out",
c  and is echoed on the terminal.
c
       if ( (char2 .eq. 'c') .or. (char2 .eq. 'C') ) then
         write (6, 2010) line
         go to 30
       endif
c
c      If column 2 contains a '$' sentinal, then a new namelist has
c  begun without the previous one being closed.  Issue error message.
c
       if (char3 .eq. '$') call nlsdac15 ( oldline, 72, 13 )
c
c      New "line" is OK, resume scanning.
c
       j1 = 3
       go to 10
c
c      Error: premature end of file.
c
40     continue
       call nlsdac15 ( line, 72, 12 )
c
c      Error: problem reading input deck
c
50     continue
       call nlsdac15 ( blank8, 0, 7 )
c
c-----------------------------------------------------------------------
c----------------------- Read format statements ------------------------
c-----------------------------------------------------------------------
c
1010   format(a72)
c
c-----------------------------------------------------------------------
c---------------------- Write format statements ------------------------
c-----------------------------------------------------------------------
c
2010   format(a72)
c
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////              N L S D A C 1 7              \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////              N L S D A C 1 8              \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac18 ( line, jbeg, word32 )
c
c    dac:namelist.nlsdac18 <---- checks for valid integer representation
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine calls nlsdac15 (which issues fatal error
c  messages) if the input character string ("word32") does not represent
c  an integer.
c
c  EXTERNALS:  NLSDAC15
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*1   char1
       character*32  word32
       character*72  line
       integer       jbeg    , ineg    , ipos    , idig    , iblnk
     1             , i       , ibeg    , iend    , j       , jpos
c
       character*1   ch      (  10)
c
c      Data statements
c
       data          ch       /'0','1','2','3','4','5','6','7','8','9'/
c
c      External statements
c
       external      nlsdac15
c
c-----------------------------------------------------------------------
c
c      Initialise parameters.
c
       ineg  = 0
       ipos  = 0
       idig  = 0
       iblnk = 1
c
c      Find first non-blank character.
c
       do 10 i=1,32
         ibeg  = i
         char1 = word32(i:i)
         if (char1 .ne. ' ') go to 20
10     continue
c
c      "word32" is blank.  Issue an error message.
c
       call nlsdac15 ( line, jbeg, 11 )
c
c      Scan word32 up to the next blank.  An error message is issued if:
c
c   1. more than one sign (+ or -) is encountered, or
c   2. a sign is embedded in the integer, or
c   3. if the integer is found to have more than 15 digits, or
c   4. if a character other than +, -, or a digit is found.
c
20     continue
       do 40 i=ibeg,32
         jpos  = jbeg + i
         iend  = i
         char1 = word32(i:i)
         if (char1 .eq. ' ') go to 50
         if (char1 .eq. '-') then
           ineg = ineg + 1
           if ( (ineg .gt. 1) .or. (ipos .gt. 0) )
     1       call nlsdac15 ( line, jpos, 4 )
           if (idig .gt. 0) call nlsdac15 ( line, jpos, 4 )
           go to 40
         endif
         if (char1 .eq. '+') then
           ipos = ipos + 1
           if ( (ipos .gt. 1) .or. (ineg .gt. 0) )
     1       call nlsdac15 ( line, jpos, 4 )
           if (idig .gt. 0) call nlsdac15 ( line, jpos, 4 )
           go to 40
         endif
         do 30 j=1,10
           if (char1 .eq. ch(j)) then
             idig = idig + 1
             if (idig .gt. 15) call nlsdac15 ( line, jpos, 15 )
             go to 40
           endif
30       continue
         call nlsdac15 ( line, jpos, 4 )
40     continue
c
c      First blank after data entry is found.
c
50     continue
c
c      If there are no digits after sign, issue error message.
c
       if (idig .eq. 0) call nlsdac15 ( line, jpos, 4 )
c
c      Scan rest of "word32" and issue error message if a non-blank
c  character is found.
c
       if (iend .lt. 32) then
         do 60 i=iend,32
           char1 = word32(i:i)
           jpos  = jbeg + i
           if (char1 .ne. ' ') call nlsdac15 ( line, jpos, 4 )
60       continue
       endif
       return
c
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////              N L S D A C 1 8              \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////              N L S D A C 1 9              \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac19 ( word32, ival )
c
c    dac:namelist.nlsdac19 <----- translates character string to integer
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine translates a character string to an
c  integer.  It is assumed that the character string is a valid
c  representation of an integer, and is error-free (i.e., NLSDAC18 has
c  been called).
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*1   char1
       character*32  word32
       integer       ival    , ichar1  , ifact   , i
c
       character*1   ch      (  10)
c
c      Data statements
c
       data          ch       /'0','1','2','3','4','5','6','7','8','9'/
c
c-----------------------------------------------------------------------
c
       ichar1 = 32
       ifact  = 1
       ival   = 0
10     continue
       char1 = word32(ichar1:ichar1)
       if (char1 .ne. ' ') then
         do 20 i=1,10
           if (char1 .eq. ch(i)) go to 30
20       continue
         if (char1 .eq. '-') then
           ival = - ival
           return
         endif
30       continue
         ival  = ival + ifact * (i - 1)
         ifact = ifact * 10
       endif
       ichar1 = ichar1 - 1
       if (ichar1 .gt. 0) go to 10
       return
c
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////              N L S D A C 1 9              \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////              N L S D A C 2 0              \\\\\\\\\\
c
c=======================================================================
c
c
       subroutine nlsdac20 ( ival, word32 )
c
c    dac:namelist.nlsdac20 <----- translates integer to character string
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine translates an integer into a left-justified
c  character string.
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*32  word32  , blank32
       integer       ival    , j       , jj      , jbeg    , jend
     1             , ilog
       real          x       , x1      , xlog
c
       character*1   ch      (  10)
c
c      Data statements
c
       data          ch       /'0','1','2','3','4','5','6','7','8','9'/
       data          blank32  /'                                '/
c
c-----------------------------------------------------------------------
c
       word32 = blank32
       if (ival .lt. 0) then
         jbeg = 2
         ival = - ival
         word32(1:1) = '-'
       else
         jbeg = 1
       endif
       x = float(ival)
       if (ival .eq. 0) then
         ilog = 0
       else
         xlog = alog10(x)
         ilog = ifix (xlog + .000001)
       endif
       jend = jbeg + ilog
       if (jend .ge. 32) jend = 32
       do 10 j=jbeg,jend
         x1 = x / 10.0**ilog
         jj = ifix ( x1 + .000001 )
         word32(j:j) = ch(jj+1)
         x  = x - float(jj) * 10**ilog
         ilog = ilog - 1
10     continue
       return
c
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////              N L S D A C 2 0              \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 2 1             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac21 ( word32, rval )
c
c    dac:namelist.nlsdac21 <-------- translates character string to real
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine translates a character string to a real
c  constant.
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*1   char1
       character*32  word32
       integer       i       , esign   , ipnt    , npnt    , more
     1             , iblnk   , iman    , iexp    , j
       real          rval    , msign   , term    , fact
c
       character*1   ch      (  10)
       integer       exponent(   4), mantissa(  16)
c
c      Data statements
c
       data          ch       /'0','1','2','3','4','5','6','7','8','9'/
c
c-----------------------------------------------------------------------
c
c      Initialise parameters.
c
       do 10 i=1,4
         exponent(i) = 0
10     continue
       do 20 i=1,16
         mantissa(i) = 0
20     continue
       msign = 1.0
       esign = 1
       ipnt  = 0
       npnt  = 0
       more  = 1
       iblnk = 0
       iman  = 0
       iexp  = 0
c
c      Scan "word32".  It is assumed that "word32" has already been
c  determined to be error-free, and that it is a valid character
c  representation of a real value (i.e., NLSDAC23 has been called).
c
       do 40 i=1,32
         char1 = word32(i:i)
         if (char1 .eq. ' ') then
           iblnk = iblnk + 1
           go to 40
         endif
         if (char1 .eq. '-') then
           if (more .eq. 1) msign = -1.0
           if (more .eq. 2) esign = -1
           go to 40
         endif
         if (char1 .eq. '+') go to 40
         if (char1 .eq. '.') then
           npnt = 1
           ipnt = i - 2 - iblnk
           if (msign .eq. -1.0) ipnt = ipnt - 1
           go to 40
         endif
         if ( (char1 .eq. 'e') .or. (char1 .eq. 'd') ) then
           more = 2
           if (npnt .eq. 0) then
             npnt = 1
             ipnt = i - 2 - iblnk
             if (msign .eq. -1.0) ipnt = ipnt - 1
             if (ipnt .lt. 0) ipnt = 0
           endif
           go to 40
         endif
         do 30 j=1,10
           if (char1 .eq. ch(j)) then
             if (more .eq. 1) then
               iman = iman + 1
               mantissa(iman) = j - 1
             endif
             if (more .eq. 2) then
               iexp = iexp + 1
               exponent(iexp) = j - 1
             endif
             go to 40
           endif
30       continue
40     continue
       if (npnt .eq. 0) ipnt = iman - 1
c
c      Evaluate the mantissa.
c
       rval = 0.0
       if (iman .gt. 0) then
         do 50 i=1,iman
           term = float(mantissa(i)) * 10.0**ipnt
           rval = rval + term
           ipnt = ipnt - 1
50       continue
       else
         rval = 1.0
       endif
       rval = rval * msign
c
c      Evaluate the exponential factor.
c
       fact = 1.0
       if (iexp .gt. 0) then
         do 60 i=1,iexp
           ipnt = iexp - i
           fact = fact * 10.0 ** (exponent(i) * 10.0**ipnt)
60       continue
       endif
       if (esign .eq. -1) fact = 1.0 / fact
c
c      Return real representation of character string.
c
       rval = rval * fact
       return
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 2 1             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 2 2             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac22 ( line, jbeg, word32 )
c
c    dac:namelist.nlsdac22 <---- checks for valid logical representation
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine calls nlsdac15 (which issues fatal error
c  messages) if the input character string ("word32") does not represent
c  a valid logical value.
c
c  EXTERNALS:  NLSDAC15
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*32  word32
       character*72  line
       integer       jbeg
c
c      External statements
c
       external      nlsdac15
c
c-----------------------------------------------------------------------
c
       if ( (word32(1:8) .ne. '.true.  ') .and.
     1      (word32(1:8) .ne. '.t.     ') .and.
     2      (word32(1:8) .ne. '.false. ') .and.
     3      (word32(1:8) .ne. '.f.     ') )
     4   call nlsdac15 ( line, jbeg, 5 )
c
       return
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 2 2             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 2 3              \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac23 ( line, jbeg, word32 )
c
c    dac:namelist.nlsdac18 <-- determines if character string is a valid
c                                         representation of a real value
c                                                              may, 1989
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine calls nlsdac15 (which issues various fatal
c  error messages) if the input character string ("word32") does not
c  represent a real value.
c
c  EXTERNALS:  NLSDAC15
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*1   char1
       character*32  word32
       character*72  line
       integer       jbeg    , mneg    , eneg    , mpos    , epos
     1             , iexp    , iman    , ipnt    , more    , iblnk
     2             , i       , ibeg    , iend    , j       , jpos
c
       character*1   ch      (  10)
c
c      Data statements
c
       data          ch       /'0','1','2','3','4','5','6','7','8','9'/
c
c      External statements
c
       external      nlsdac15
c
c-----------------------------------------------------------------------
c
c      Initialise parameters.
c
       mneg  = 0
       eneg  = 0
       mpos  = 0
       epos  = 0
       iexp  = 0
       iman  = 0
       ipnt  = 0
       more  = 1
       iblnk = 1
c
c      Find first non-blank character.
c
       do 10 i=1,32
         ibeg  = i
         char1 = word32(i:i)
         if (char1 .ne. ' ') go to 20
10     continue
c
c      "word32" is blank.  Issue an error message.
c
       call nlsdac15 ( line, jbeg, 11 )
c
c      Scan word32 up to the next blank.  An error message is issued if:
c
c   1. more than one sign (+ or -) is encountered in the mantissa, or
c   2. a sign is embedded in the mantissa, or
c   3. more than one sign (+ or -) is encountered in the exponent, or
c   4. a sign is embedded in the exponent, or
c   5. if more than one decimal point is encountered in the mantissa, or
c   6. if a decimal point is encountered in the exponent, or
c   7. if more than one exponential sentinal ("e" or "d") is found, or
c   8. if the mantissa is found to have more than 16 digits, or
c   9. if the mantissa is found to have more than  4 digits, or
c  10. if a character other than +, -, ., e, d, or a digit is found.
c
20     continue
       do 40 i=ibeg,32
         jpos  = jbeg + i
         iend  = i
         char1 = word32(i:i)
         if (char1 .eq. ' ') go to 50
         if ( (char1 .eq. '-') .and. (more .eq. 1) ) then
           mneg = mneg + 1
           if ( (mneg .gt. 1) .or. (mpos .gt. 0) )
     1       call nlsdac15 ( line, jpos, 4 )
           if (iman .gt. 0) call nlsdac15 ( line, jpos, 4 )
           go to 40
         endif
         if ( (char1 .eq. '-') .and. (more .eq. 2) ) then
           eneg = eneg + 1
           if ( (eneg .gt. 1) .or. (epos .gt. 0) )
     1       call nlsdac15 ( line, jpos, 4 )
           if (iexp .gt. 0) call nlsdac15 ( line, jpos, 4 )
           go to 40
         endif
         if ( (char1 .eq. '+') .and. (more .eq. 1) ) then
           mpos = mpos + 1
           if ( (mpos .gt. 1) .or. (mneg .gt. 0) )
     1       call nlsdac15 ( line, jpos, 4 )
           if (iman .gt. 0) call nlsdac15 ( line, jpos, 4 )
           go to 40
         endif
         if ( (char1 .eq. '+') .and. (more .eq. 2) ) then
           epos = epos + 1
           if ( (epos .gt. 1) .or. (eneg .gt. 0) )
     1       call nlsdac15 ( line, jpos, 4 )
           if (iexp .gt. 0) call nlsdac15 ( line, jpos, 4 )
           go to 40
         endif
         if ( (char1 .eq. '.') .and. (more .eq. 1) ) then
           ipnt = ipnt + 1
           iman = iman + 1
           if (ipnt .gt. 1) call nlsdac15 ( line, jpos, 4 )
           go to 40
         endif
         if ( (char1 .eq. '.') .and. (more .eq. 2) ) then
           call nlsdac15 ( line, jpos, 4 )
         endif
         if ( (char1 .eq. 'e') .or. (char1 .eq. 'd') ) then
           if (more .eq. 2) call nlsdac15 ( line, jpos, 4 )
           more = 2
           go to 40
         endif
         do 30 j=1,10
           if (char1 .eq. ch(j)) then
             if (more .eq.  1) iman = iman + 1
             if (iman .gt. 16) call nlsdac15 ( line, jpos, 15 )
             if (more .eq.  2) iexp = iexp + 1
             if (iexp .gt.  3) call nlsdac15 ( line, jpos, 14 )
             go to 40
           endif
30       continue
         call nlsdac15 ( line, jpos, 4 )
40     continue
c
c      First blank after data entry is found.
c
50     continue
c
c      If there are no digits after sign and/or decimal point, issue
c  error message.
c
       iman = iman - ipnt
       if (iman .eq. 0) call nlsdac15 ( line, jpos, 4 )
c
c      Scan rest of "word32" and issue error message if a non-blank
c  character is found.
c
       if (iend .lt. 32) then
         do 60 i=iend,32
           char1 = word32(i:i)
           jpos  = jbeg + i
           if (char1 .ne. ' ') call nlsdac15 ( line, jpos, 4 )
60       continue
       endif
       return
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 2 3             \\\\\\\\\\
c
c=======================================================================
c
c
c=======================================================================
c
c     \\\\\\\\\\     B E G I N   S U B R O U T I N E     //////////
c     //////////             N L S D A C 2 4             \\\\\\\\\\
c
c=======================================================================
c
       subroutine nlsdac24 ( lunit, cname, namelist, nmax )
c
c    dac:namelist.nlsdac24 <---- dumps namelist values to specified unit
c                                                         february, 1991
c
c      Written by: David Clarke
c      Modified 1:
c
c  PURPOSE:  This subroutine dumps a formatted summary of all user-
c  specified namelist parameters in the namelist "cname" to the logical
c  unit "lunit".
c
c  EXTERNALS:  NONE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       logical       lval
       character*1   char1   , attr    , type
       character*4   jbeg    , jend    , kbeg    , kend
       character*8   cname   , word8   , temp8   , nword8
       character*25  var
       character*32  blank32
       character*512 cval
       integer       lunit   , nmax    , mult    , i       , ilist
     1             , ival    , lbeg    , lend    , lran    , j
     2             , j1      , j2      , mbeg    , mend
       real          rval
c
       character*32  namelist(   2,   *)
c
c      Data statements
c
       data          blank32  / '                                ' /
c
c      External statements
c
       external      nlsdac19, nlsdac21
c
c-----------------------------------------------------------------------
c
       mult = 0
       lbeg = 1
       do 10 j=1,16
         j2 = 32 * j
         j1 = j2 - 31
         cval(j1:j2) = blank32
10     continue
c
c      First line to output unit.
c
       write (lunit, 2010) cname
c
c      Read namelist entries, and write contents to output unit.
c
       do 100 i=1,nmax
         ilist = i
         if (namelist(2,ilist) .eq. blank32) go to 100
         char1 = namelist(1,ilist)(31:31)
         if (char1 .eq. '*') go to 100
         word8 = namelist(1,ilist)( 1: 8)
         if (word8 .eq. '        ') go to 110
         namelist(1,ilist)(31:31) = '*'
20       continue
         var   = blank32(1:25)
         attr  = namelist(1,ilist)(30:30)
         type  = namelist(1,ilist)(32:32)
c
c      Representation of variable name (contents of "namelist(1,*)")
c  depends on type of variable (ie, scalar, vector, or 2-D array).
c
         if (type .ne. 's') then
           jbeg = namelist(1,ilist)( 9:12)
           jend = namelist(1,ilist)(13:16)
           if (jend .eq. '    ') jend = jbeg
           kbeg = namelist(1,ilist)(17:20)
           kend = namelist(1,ilist)(21:24)
           if (kend .eq. '    ') kend = kbeg
           if (type .eq. 'v') write (var, 2020) word8, jbeg, jend
           if (type .eq. 'a') write (var, 2030) word8, jbeg, jend
     1                                               , kbeg, kend
         else
           write (var, 2040) word8
         endif
c
c      Representation of variable value (contents of "namelist(2,*)")
c  depends on attribute (ie, integer, real, logical, or character).
c
c  Case:  integer
c
         if (attr .eq. 'i') then
           call nlsdac19 ( namelist(2,ilist), ival )
           write (lunit, 2050) var, ival
         endif
c
c  Case:  real
c
         if (attr .eq. 'r') then
           call nlsdac21 ( namelist(2,ilist), rval )
           write (lunit, 2060) var, rval
         endif
c
c  Case:  logical
c
         if (attr .eq. 'l') then
           temp8 = namelist(2,ilist)(1:8)
           if (temp8 .eq. '.true.  ') lval = .true.
           if (temp8 .eq. '.t.     ') lval = .true.
           if (temp8 .eq. '.false. ') lval = .false.
           if (temp8 .eq. '.f.     ') lval = .false.
           if (      lval) write (lunit, 2070) var
           if (.not. lval) write (lunit, 2080) var
         endif
c
c  Case:  character
c
c  For character strings, there is the possibility that the string is
c  stored in pieces throughout "namelist(2,*)".  Thus, delay writing
c  the entry to the output unit until all the character string can be
c  gathered.
c
         if (attr .eq. 'c') then
           if (namelist(2,ilist)(32:32) .eq. '&') then
             mult = 1
             lran = ichar ( namelist(2,ilist)(31:31) )
             lend = lbeg + lran - 1
             cval(lbeg:lend) = namelist(2,ilist)(1:lran)
             lbeg = lend + 1
           else
             do 50 lran=31,1,-1
               if (namelist(2,ilist)(lran:lran) .ne. ' ') go to 60
50           continue
60           continue
             lend = lbeg + lran - 1
             cval(lbeg:lend) = namelist(2,ilist)(1:lran)
             if (lend .le. 50) then
               write (lunit, 2090) var, cval(1:lend)
             else
               mbeg = 1
               mend = 50
               write (lunit, 2100) var, cval(mbeg:mend)
70             continue
               mbeg = mend + 1
               mend = mend + 50
               if (mend .ge. lend) then
                 write (lunit, 2110) cval(mbeg:lend)
               else
                 write (lunit, 2120) cval(mbeg:mend)
                 go to 70
               endif
             endif
             mult = 0
             lbeg = 1
             do 80 j=1,16
               j2 = 32 * j
               j1 = j2 - 31
               cval(j1:j2) = blank32
80           continue
           endif
         endif
c
c      Check the rest of "namelist(1,*)" for any other variable entries
c  with the same variable name.  Such entries may exist for vectors and
c  arrays of all attributes, and scalars for character variables.  An
c  asterisk is put (temporarily) in column 31 of "namelist(1,ilist)" so
c  that this entry will not be read again.
c
         if (ilist .ne. nmax) then
           do 90 j=ilist+1,nmax
             nword8  = namelist(1,j)( 1: 8)
             if (nword8 .eq. '        ') go to 100
             if (nword8 .eq. word8) then
               namelist(1,j)(31:31) = '*'
               ilist = j
               go to 20
             endif
90         continue
         endif
100    continue
110    continue
c
c      Last line to output unit.
c
       write (lunit, 2130) cname
c
c      Replace asterisks put in column 31 of "namelist(1,*)" with
c  blanks.
c
       do 120 i=1,nmax
         if (namelist(1,i)(1:8) .eq. '        ') return
         namelist(1,i)(31:31) = ' '
120    continue
c
       return
c
c-----------------------------------------------------------------------
c---------------------- Write format statements ------------------------
c-----------------------------------------------------------------------
c
2010   format(/,'======================================================'
     1       ,'> BEGIN NAMELIST: ',a8)
2020   format(a8,'(',a3,':',a3,')')
2030   format(a8,'(',a3,':',a3,',',a3,':',a3,')')
2040   format(a8)
2050   format(a25,' =',i10)
2060   format(a25,' =',1pg20.12)
2070   format(a25,' = .true.')
2080   format(a25,' = .false.')
2090   format(a25,' = "',a,'"')
2100   format(a25,' = "',a)
2110   format(29x,a,'"')
2120   format(29x,a)
2130   format('========================================================'
     1       ,'> END NAMELIST: ',a8)
c
       end
c
c=======================================================================
c
c     \\\\\\\\\\       E N D   S U B R O U T I N E       //////////
c     //////////             N L S D A C 2 4             \\\\\\\\\\
c
c=======================================================================
c
c
