#!/bin/csh
#
# HYDRO_SCALED [-c] [NZ1] [NZ2] [NZ3]
#
# Compile ZEUS-3D and optionally run the blast hydro test problem 
# on various grids.
#
# OPTIONS
#      -c     compile only.  
#      NZ*    number of zones in the "*" direction along a tile.
#             If only NZ1 is present, the tile is assumed cubical.
#
# Executables and input files for each grid are saved.
#
# RAF 5/20/97
#...................................................................
#
# Check for compile-only flag
#
set RUN = YES
if ($#argv > 0) then
  if ("$1" == "-c") then
    set RUN = NO
    shift
  endif
  if ($#argv > 0) then
    set NZ1 = $1
    shift
  else
    set NZ1 = 32
    set NZ2 = 32
    set NZ3 = 32
  endif
  if ($#argv > 0) then
    set NZ2 = $1
    shift
  else
    set NZ2 = $NZ1
    set NZ3 = $NZ1
  endif
  if ($#argv > 0) then
    set NZ3 = $1
    shift
  else
    set NZ3 = $NZ2
  endif
endif
#
# Identify this calling script to HYDRO_BM.s
#
setenv SCALED ""
#
# Disable interactive display/edit in zeus34.s
#
setenv ENVIRONMENT BATCH
#
# Define the output file
#
setenv ID $$
set DATA_DIR = "${HOME}/zeus3d_BM"
if (! -e $DATA_DIR) mkdir $DATA_DIR
setenv ZBM_Z "${DATA_DIR}/ZBM_B_HY_${NZ1}_${NZ2}_${NZ3}.${ID}"
#
# Set variables to turn off NCAR, HDF
#
setenv HDF ""
setenv NCAR_LIB "ncard"
if (! $?HOST_NAME) setenv HOST_NAME `hostname | sed -e 's/\..*//'`
switch ($HOST_NAME)
case mario:
  setenv LDR segldr
  set N_THREADS_LIM = 1
  breaksw
case modi1:
case modi2:
case modi3:
case modi4:
  setenv LDR "f77 -64 -r10000"
  set N_THREADS_LIM = 32
  breaksw
case bragi:
case freya:
case odin:
case thor:
case sif:
  setenv LDR f77
  set N_THREADS_LIM = 16
  breaksw
case magna:
  setenv LDR "f77 -64 -r10000"
  set N_THREADS_LIM = 32
  breaksw
case loki:
  setenv LDR f77
  set N_THREADS_LIM = 12
  breaksw
case lena:
  setenv LDR /usr/convex/bin/fc
  set N_THREADS_LIM = 16
  breaksw
case ella:
  setenv LDR /opt/fortran/bin/f77
  set N_THREADS_LIM = 16
  breaksw
case tuvok:
  setenv LDR /opt/fortran/bin/f77
  set N_THREADS_LIM = 16
  breaksw
case billie:
  setenv LDR /opt/fortran/bin/f77
  set N_THREADS_LIM = 32
  breaksw
case dax:
  setenv LDR /opt/fortran/bin/f77
  set N_THREADS_LIM = 64
  breaksw
endsw
#
set IGRD = 1
while ($IGRD <= 7)
#
# For each grid
#
  if (-e chgzeus)    \rm chgzeus
  if (-e inzeus)     \rm inzeus

  switch ($IGRD)
    case 1:
      @ N1 = $NZ1 * 1
      @ N2 = $NZ2 * 1
      @ N3 = $NZ3 * 1
      set N_THREADS_MAX = 1
      setenv N_THREADS_MIN 1
      breaksw
    case 2:
      @ N1 = $NZ1 * 1
      @ N2 = $NZ2 * 1
      @ N3 = $NZ3 * 2
      set N_THREADS_MAX = 2
      setenv N_THREADS_MIN 2
      breaksw
    case 3:
      @ N1 = $NZ1 * 1
      @ N2 = $NZ2 * 1
      @ N3 = $NZ3 * 4
      set N_THREADS_MAX = 4
      setenv N_THREADS_MIN 4
      breaksw
    case 4:
      @ N1 = $NZ1 * 1
      @ N2 = $NZ2 * 1
      @ N3 = $NZ3 * 8
      set N_THREADS_MAX = 8
      setenv N_THREADS_MIN 8
      breaksw
    case 5:
      @ N1 = $NZ1 * 1
      @ N2 = $NZ2 * 1
      @ N3 = $NZ3 * 16
      set N_THREADS_MAX = 16
      setenv N_THREADS_MIN 16
      breaksw
    case 6:
      @ N1 = $NZ1 * 1
      @ N2 = $NZ2 * 1
      @ N3 = $NZ3 * 32
      set N_THREADS_MAX = 32
      setenv N_THREADS_MIN 32
      breaksw
    case 7:
      @ N1 = $NZ1 * 1
      @ N2 = $NZ2 * 1
      @ N3 = $NZ3 * 64
      set N_THREADS_MAX = 64
      setenv N_THREADS_MIN 64
      breaksw
  endsw
#  switch ($IGRD)
#    case 1:
#      @ N1 = $NZ1 * 1
#      @ N2 = $NZ2 * 1
#      @ N3 = $NZ3 * 1
#      set N_THREADS_MAX = 1
#      setenv N_THREADS_MIN 1
#      breaksw
#    case 2:
#      @ N1 = $NZ1 * 2
#      @ N2 = $NZ2 * 1
#      @ N3 = $NZ3 * 1
#      set N_THREADS_MAX = 2
#      setenv N_THREADS_MIN 2
#      breaksw
#    case 3:
#      @ N1 = $NZ1 * 4
#      @ N2 = $NZ2 * 1
#      @ N3 = $NZ3 * 1
#      set N_THREADS_MAX = 4
#      setenv N_THREADS_MIN 4
#      breaksw
#    case 4:
#      @ N1 = $NZ1 * 8
#      @ N2 = $NZ2 * 1
#      @ N3 = $NZ3 * 1
#      set N_THREADS_MAX = 8
#      setenv N_THREADS_MIN 8
#      breaksw
#    case 5:
#      @ N1 = $NZ1 * 16
#      @ N2 = $NZ2 * 1
#      @ N3 = $NZ3 * 1
#      set N_THREADS_MAX = 16
#      setenv N_THREADS_MIN 16
#      breaksw
#    case 6:
#      @ N1 = $NZ1 * 32
#      @ N2 = $NZ2 * 1
#      @ N3 = $NZ3 * 1
#      set N_THREADS_MAX = 32
#      setenv N_THREADS_MIN 32
#      breaksw
#    case 7:
#      @ N1 = $NZ1 * 64
#      @ N2 = $NZ2 * 1
#      @ N3 = $NZ3 * 1
#      set N_THREADS_MAX = 64
#      setenv N_THREADS_MIN 64
#      breaksw
#  endsw
  setenv NZX1 $N1
  setenv NZX2 $N2
  setenv NZX3 $N3
  set  GRID = "${NZX1}x${NZX2}x${NZX3}"
  set UGRID = "${NZX1}_${NZX2}_${NZX3}"
  set INP = "in_b_${UGRID}"
#
# Do HY only
#
  set EXE = "xz_b_hy_${UGRID}"
  if (-e $EXE) then
    cp $EXE xzeus34
    cp $INP inzeus
  else
#
# Compile using all defaults
#
    echo "I need to compile ZEUS-3D to make $EXE"
    echo ""
    if (! -e blast_HY) then
      echo "However, I need the setup block blast_HY on our server in"
      echo "/afs/ncsa/projects/lca/codes/zeus3d/tests/blast/XYZ"
      exit 1
    endif
    if (-e /usr/afsws/bin/klog && \
      ! -r /afs/ncsa/projects/lca/codes/zeus3d/source/zeus34.s) \
      /usr/afsws/bin/klog -pr zeus3d -pa DDDVets -cell ncsa.uiuc.edu
    zeus34.s blast_HY
    cp xzeus34 $EXE
    cp inzeus $INP
  endif
#
# Run N_THREADS_MAX experiments
#
  if ($RUN == YES && $N_THREADS_MAX <= $N_THREADS_LIM) HYDRO_BM.s $N_THREADS_MAX 0 $GRID
#
# Increment counter
#
  @ IGRD = $IGRD + 1
end
#
# Finish up
#
cat $ZBM_Z
\rm -f NORM.${ID}
exit
